import 'dart:math';

import 'package:flutter/material.dart';
import 'package:tetrisdual/provider/data.dart';

class Counter {
  // Current counter
  bool _match = false; // Does this new tetrimino touch an other tetrimino of same player
  int _lines = 0; // Count lines fully filled by this new tetrimino
  int _holes = 0; // Count non fillable holes caused by this new tetrimino

  // Points definitions
  int _base = 50;
  int _pointsIfMatch = 50;
  int _pointsPerLine = 60;
  int _pointsPerHole = -10;

  Color categoryIconColor = Colors.green;
  Color buttonIconColor = Colors.blue;
  double iconSize = 30.0;

  TableRow spacer() {
    double height = 7;
    return TableRow(children: [
      SizedBox(height: height),
      SizedBox(height: height),
      SizedBox(height: height),
      SizedBox(height: height)
    ]);
  }

  Widget buildCounterWidget(Data myProvider) {
    return Container(
      child: Table(
        children: [
          buildMatchWidget(myProvider),
          spacer(),
          buildLinesWidget(myProvider),
          spacer(),
          buildHolesWidget(myProvider),
          spacer(),
        ],
      ),
    );
  }

  TableRow buildMatchWidget(Data myProvider) {
    return TableRow(
      children: [
        Icon(
          Icons.join_full,
          color: categoryIconColor,
          size: iconSize,
        ),
        IconButton(
          padding: EdgeInsets.zero,
          constraints: BoxConstraints(),
          icon: Icon(
            Icons.remove,
            color: buttonIconColor,
            size: iconSize,
          ),
          onPressed: () {
            _match = false;
            myProvider.redraw();
          },
        ),
        Center(
          child: Icon(
            _match ? Icons.radio_button_checked : Icons.radio_button_unchecked,
            color: categoryIconColor,
            size: iconSize,
          ),
        ),
        IconButton(
          padding: EdgeInsets.zero,
          constraints: BoxConstraints(),
          icon: Icon(
            Icons.add,
            color: buttonIconColor,
            size: iconSize,
          ),
          onPressed: () {
            _match = true;
            myProvider.redraw();
          },
        ),
      ],
    );
  }

  TableRow buildLinesWidget(Data myProvider) {
    return TableRow(
      children: [
        Icon(
          Icons.table_rows,
          color: categoryIconColor,
          size: iconSize,
        ),
        IconButton(
          padding: EdgeInsets.zero,
          constraints: BoxConstraints(),
          icon: Icon(
            Icons.remove,
            color: buttonIconColor,
            size: iconSize,
          ),
          onPressed: () {
            _lines = max(_lines - 1, 0);
            myProvider.redraw();
          },
        ),
        Center(
          child: Text(
            _lines.toString(),
            style: TextStyle(
              fontFamily: 'Blocks',
              fontSize: 50,
            ),
          ),
        ),
        IconButton(
          padding: EdgeInsets.zero,
          constraints: BoxConstraints(),
          icon: Icon(
            Icons.add,
            color: buttonIconColor,
            size: iconSize,
          ),
          onPressed: () {
            _lines = min(_lines + 1, 4);
            myProvider.redraw();
          },
        ),
      ],
    );
  }

  TableRow buildHolesWidget(Data myProvider) {
    return TableRow(
      children: [
        Icon(
          Icons.check_box_outline_blank,
          color: categoryIconColor,
          size: iconSize,
        ),
        IconButton(
          padding: EdgeInsets.zero,
          constraints: BoxConstraints(),
          icon: Icon(
            Icons.remove,
            color: buttonIconColor,
            size: iconSize,
          ),
          onPressed: () {
            _holes = max(_holes - 1, 0);
            myProvider.redraw();
          },
        ),
        Center(
          child: Text(
            _holes.toString(),
            style: TextStyle(
              fontFamily: 'Blocks',
              fontSize: 50,
            ),
          ),
        ),
        IconButton(
          padding: EdgeInsets.zero,
          constraints: BoxConstraints(),
          icon: Icon(
            Icons.add,
            color: buttonIconColor,
            size: iconSize,
          ),
          onPressed: () {
            _holes = min(_holes + 1, 4);
            myProvider.redraw();
          },
        ),
      ],
    );
  }

  int computePoints() {
    return _base +
        (_match ? _pointsIfMatch : 0) +
        _lines * _pointsPerLine +
        _holes * _pointsPerHole;
  }

  void reset() {
    _match = false;
    _lines = 0;
    _holes = 0;
  }
}
