import 'dart:math';

import 'package:flutter/material.dart';
import 'package:tetrisdual/entity/counter.dart';
import 'package:tetrisdual/layout/board_painter.dart';
import 'package:tetrisdual/provider/data.dart';

class Player {
  Player(this.playerId);
  int playerId;

  int _score = 0;
  int _currentTetrimino = 0;
  Counter _counter = new Counter();

  Widget buildTetriminoWidget(Data myProvider, double width) {
    return Container(
      child: GestureDetector(
        onTapUp: (details) {
          if (playerId == myProvider.getCurrentPlayer().playerId) {
            pickRandomTetrimino();
            myProvider.redraw();
          }
        },
        child: Container(
          child: CustomPaint(
            size: Size(width, width),
            willChange: false,
            painter: BoardPainter(_currentTetrimino),
            isComplex: true,
            key: Key(_currentTetrimino.toString()),
          ),
        ),
      ),
    );
  }

  Widget buildManagerWidget(Data myProvider) {
    List<Widget> items = [];

    if (myProvider.currentPlayer == playerId) {
      items.add(_counter.buildCounterWidget(myProvider));
      items.add(buildSubmitWidget(myProvider));
    }

    return Expanded(
      child: Container(
        margin: EdgeInsets.all(5),
        child: Column(
          mainAxisAlignment: MainAxisAlignment.start,
          crossAxisAlignment: CrossAxisAlignment.center,
          children: items,
        ),
      ),
    );
  }

  Widget buildSubmitWidget(Data myProvider) {
    double fontSize = 70;

    return Container(
      decoration: BoxDecoration(
        border: Border(
          top: BorderSide(
            color: Colors.black,
            width: 2,
          ),
        ),
      ),
      padding: EdgeInsets.only(
        top: 5,
      ),
      child: Row(
        mainAxisAlignment: MainAxisAlignment.end,
        crossAxisAlignment: CrossAxisAlignment.center,
        children: [
          Text(
            '+' + _counter.computePoints().toString(),
            style: TextStyle(
              fontFamily: 'Blocks',
              fontSize: fontSize,
              fontWeight: FontWeight.bold,
            ),
          ),
          SizedBox(
            width: 10,
          ),
          IconButton(
            padding: EdgeInsets.zero,
            constraints: BoxConstraints(),
            icon: Icon(
              Icons.done_all,
              color: Colors.orange,
              size: fontSize / 2.5,
            ),
            onPressed: () {
              _score = _score + _counter.computePoints();
              _counter.reset();
              myProvider.toggleCurrentPlayer();
            },
          ),
          SizedBox(
            width: 10,
          ),
        ],
      ),
    );
  }

  Widget buildPlayerBoard(Data myProvider, double screenWidth, bool isActive) {
    double tetriminoBlockWidth = screenWidth / 2.3;
    Color borderColor = isActive ? Colors.greenAccent : Colors.blueGrey;
    double borderWidth = 10;

    return Column(
      mainAxisAlignment: MainAxisAlignment.center,
      crossAxisAlignment: CrossAxisAlignment.end,
      children: [
        Text(
          _score.toString(),
          style: TextStyle(
            fontFamily: 'Blocks',
            fontSize: 130,
            fontWeight: FontWeight.bold,
          ),
        ),
        Container(
          decoration: BoxDecoration(
            border: Border.all(
              color: borderColor,
              width: borderWidth,
            ),
          ),
          child: Row(
            mainAxisAlignment: MainAxisAlignment.center,
            crossAxisAlignment: CrossAxisAlignment.start,
            children: [
              isActive
                  ? buildTetriminoWidget(myProvider, tetriminoBlockWidth)
                  : SizedBox(
                      height: tetriminoBlockWidth,
                    ),
              buildManagerWidget(myProvider),
            ],
          ),
        )
      ],
    );
  }

  void pickRandomTetrimino() {
    // ensure new tetrimino is not same as current one
    int newTetrimino = _currentTetrimino;
    while (newTetrimino == _currentTetrimino) {
      newTetrimino = Random().nextInt(5) + 1;
    }

    _currentTetrimino = newTetrimino;
  }

  void resetTetrimino() {
    _currentTetrimino = 0;
  }

  void submitPoints() {
    _score = _score + _counter.computePoints();
    _counter.reset();
  }
}
